/*
 *	PCMCIA Smart Card Reader Driver Kernel module
 *  Copyright (C) 2005-2006  TL Sudheendran(tlsudheendran@scmmicro.co.in)

 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.

 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *	SCM Microsystems  Inc., hereby disclaims all copyright interest in the
 *	library module  written by TL Sudheendran

	Module		:	protos.h

	Author		:	SCM Microsystems

	Development History :

		Date		Version No.		Changes
		_______________________________________________
		02 Feb, 2004	2.0.1		Initial Versions
		30 Mar, 2004	3.0.1		Added support for suspend, resume
									under event handler.
		22 Nov, 2004	3.1.1		TTP issues 1359,1360
		25 Feb, 2005	3.2.1		Added support for Linux 2.6.x; TTP 1410, 2139
		17 May, 2005	4.1.1		Removed known limitations regarding surprise
									removal; Added LGPL License agreement
		10 Aug, 2006	4.2.2		Added support for kernel 2.6.16


*/


#ifndef _PROTOS_H_
#define _PROTOS_H_

// pcmcia_main.c
#include "includes.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
int Driver_Attach(
		struct pcmcia_device *p_dev
		);

void Driver_Detach(
		struct pcmcia_device *p_dev
		);

int Driver_Suspend(
		struct pcmcia_device *p_dev
		);

int Driver_Resume(
		struct pcmcia_device *p_dev
		);
#else
dev_link_t *Driver_Attach(
		void
		);

void Driver_Detach(
		dev_link_t 		*pDeviceLink
		);

void	Driver_Release(
		u_long 			arg
		);

int Event_Handler(
		event_t 		event,
		int 			priority,
		event_callback_args_t 	*args
		);

#endif

ssize_t IFDHRead(
	 	struct file 		*file,
		char 			*pucRxBuffer,
		size_t 			count,
		loff_t 			*loc
		);

DWORD LocalRead(
		DEVICE_EXTENSION	*pDeviceExtension,
		char 			*pucRxBuffer,
		size_t 			count
		);

ssize_t IFDHWrite(
		struct file 		*file,
		const char 		*pucTxBuffer,
		size_t 			count,
		loff_t 			*loc
		);

ssize_t LocalWrite(
		DEVICE_EXTENSION	*pDeviceExtension,
		const char 		*pucTxBuffer,
		size_t 			count
		);


int	IFDHIoctl(
		struct inode 		*inode,
		struct file 		*file,
		u_int 			cmd,
		u_long 			arg
		);

int	IFDHOpen(
		struct inode 		*inode,
		struct file 		*file
		);

int	IFDHClose(
		struct inode 		*inode,
		struct file 		*file
		);

void 	printdummy(
		char 			*dbg_msg, ...
		);

irqreturn_t Driver_IrqHandler(
		int 			irq,
		void 			*pDev,
		struct pt_regs 		*regs
		);

void 	WaitEvent(
		void
		);

void 	SetEvent(
		void
		);

// PcdmHdlr.c
NTSTATUS
IFDGetCardStatus (
		PDEVICE_EXTENSION 	pDeviceExtension
		);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
void tqGetCardStatus( void *arg);
#else
void tqGetCardStatus( struct work_struct *pstWorkQueue);
#endif

void SetEventWait( void *arg );

BOOLEAN
PcmciaIrqServiceRoutine (
        	PDEVICE_EXTENSION 	pDeviceExtension
        	);

DWORD PcdmCheckRegistry(
		PDEVICE_EXTENSION 	pDeviceExtension,
		PUCHAR 			pucInBuffer
		);

void Driver_SetIOBase(
		DEVICE_EXTENSION 	*pDeviceExtension,
		u_long 			iobase
		);

NTSTATUS Driver_InitLocal(
		int nInstanceCount
		);

void CardStatusCheck(
		void  			*arg
		);

// PcdmRdWr.c
NTSTATUS PcdmWrite (
		DEVICE_EXTENSION	*pDeviceExtension,
		PUCHAR			pData,
		DWORD			dwDataLen
		);

NTSTATUS PcdmWriteRevA (
        	DEVICE_EXTENSION        *pDeviceExtension,
        	PUCHAR                  pData,
        	DWORD                   dwDataLen
        	);

NTSTATUS PcdmWriteRevC (
        	PDEVICE_EXTENSION       pDeviceExtension,
	        PUCHAR                  pData,
		DWORD                   dwDataLen
        	);


NTSTATUS PcdmRead (
		DEVICE_EXTENSION 	*pDeviceExtension,
		PUCHAR			pData,
		PDWORD			pdwDataLen
		);

NTSTATUS PcdmReadRevA(
        DEVICE_EXTENSION *pDeviceExtension,
        u_char *        pData,
        PDWORD          pdwDataLen
        );

NTSTATUS PcdmReadRevC (
        PDEVICE_EXTENSION       pDeviceExtension,
        u_char *                pData,
        PDWORD                  pdwDataLen
        );

NTSTATUS PcdmWaitForDeviceReady(
		DEVICE_EXTENSION 	*pDeviceExtension
		);

int PcdmCardStatusThread(
		void * arg
		);


// ifd.c

NTSTATUS
IFDPCCardDownloadStart (
		PDEVICE_EXTENSION pDeviceExtension,
	    PUCHAR pBuffer
    	);

NTSTATUS IFDPerformPCCardDownLoad(
		PDEVICE_EXTENSION		pDeviceExtension,
		PUCHAR					pBuffer
		);

void IFDPerformPCCardErase(
		PDEVICE_EXTENSION		pDeviceExtension
		);

NTSTATUS
IFDGetInterfaceDetails (
		PDEVICE_EXTENSION		pDeviceExtension
		);

NTSTATUS
IFDSetTransportMechanism (
		PDEVICE_EXTENSION 		pDeviceExtension
		);

#endif
// End of file
